#include "../../Commun/Commun/Somme_h.h"
#include <atomic>
using namespace std;

atomic<unsigned long> g_cptObj{ 0 };

class Sommateur : public ISomme {
   atomic<unsigned long> cptref;
public:
   Sommateur() : cptref{ 1 } {
      ++g_cptObj;
   }
   ~Sommateur() {
      --g_cptObj;
   }
   unsigned long __stdcall AddRef() override {
      return ++cptref;
   }
   unsigned long __stdcall Release() override {
      if (auto n = cptref--; n != 1)
         return n - 1;
      delete this;
      return {};
   }
   HRESULT __stdcall QueryInterface(REFIID riid, void** ppv) {
      if (riid == IID_IUnknown) {
         *ppv = static_cast<IUnknown*>(this);
      } else if (riid == IID_ISomme) {
         *ppv = static_cast<ISomme*>(this);
      } else {
         *ppv = nullptr;
         return E_NOINTERFACE;
      }
      AddRef();
      return S_OK;
   }
   HRESULT __stdcall somme(int x, int y, int* resultat) override {
      *resultat = x + y;
      return S_OK;
   }
};

class FabriqueSommateur : public IClassFactory {
   atomic<unsigned long> cptref;
public:
   FabriqueSommateur() : cptref{ 1 } {
   }
   ~FabriqueSommateur() {
   }
   unsigned long __stdcall AddRef() override {
      return ++cptref;
   }
   unsigned long __stdcall Release() override {
      if (auto n = cptref--; n != 1)
         return n - 1;
      delete this;
      return {};
   }
   HRESULT __stdcall QueryInterface(REFIID riid, void** ppv) {
      if (riid == IID_IUnknown) {
         *ppv = static_cast<IUnknown*>(this);
      } else if (riid == IID_IClassFactory) {
         *ppv = static_cast<IClassFactory*>(this);
      } else {
         *ppv = nullptr;
         return E_NOINTERFACE;
      }
      AddRef();
      return S_OK;
   }
   HRESULT __stdcall LockServer(BOOL fLock) override {
      if (fLock)
         ++g_cptObj;
      else
         --g_cptObj;
      return S_OK;
   }
   HRESULT __stdcall CreateInstance(IUnknown* pUnkOuter, REFIID riid, void** ppv) {
      if (pUnkOuter)
         return CLASS_E_NOAGGREGATION;
      try {
         auto p = new Sommateur;
         auto hr = p->QueryInterface(riid, ppv);
         p->Release();
         return hr;
      } catch (...) {
         return E_OUTOFMEMORY;
      }
   }
};

#include "../../Commun/Commun/UUID_Fabriques.h"
#include "Registry.h" // RegisterServer(), UnregisterServer()
#include <string>

// Utilitaires de support au contexte
const wstring SRV_NOMFICH = L"ServeurSommateur.dll"
, SRV_DESCR = L"Composant interne de dmonstration"
, SRV_NOM = L"Composant.Sommateur.InProc"
, SRV_VERSION = L"Composant.Sommateur.InProc.1"
;

HRESULT __stdcall DllRegisterServer() {
   return RegisterServer(
      SRV_NOMFICH, CLSID_FabriqueSommateur, SRV_DESCR, SRV_NOM, SRV_VERSION
   );
}
HRESULT __stdcall DllUnregisterServer() {
   return UnregisterServer(CLSID_FabriqueSommateur, SRV_NOM, SRV_VERSION);
}

HRESULT __stdcall DllGetClassObject(REFCLSID clsid, REFIID riid, void** ppv) {
   if (clsid != CLSID_FabriqueSommateur)
      return CLASS_E_CLASSNOTAVAILABLE;
   try {
      auto p = new FabriqueSommateur;
      // riid est probablement IID_IClassFactory
      auto hr = p->QueryInterface(riid, ppv);
      p->Release();
      return hr;
   } catch (...) {
      return E_OUTOFMEMORY;
   }
}

HRESULT __stdcall DllCanUnloadNow() {
   return g_cptObj == 0 ? S_OK : S_FALSE;
}

